@@ -2,7 +2,7 @@
 #
 # GNOME Activity Journal
 #
-# Copyright © 2010 Randal Barlow
+# Copyright © 2010 Randal Barlow <email.tehk@gmail.com>
 # Copyright © 2010 Markus Korn
 # Copyright © 2010 Siegfried Gevatter <siegfried@gevatter.com>
 #
@@ -19,14 +19,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-"""
-Takes a two dementional lis store of ints and turns it into a graph based on
-the first value a int date, and the second value the number of items on that date
-where items are
-datastore = []
-datastore.append(time, nitems)
-CairoHistogram.set_datastore(datastore)
-"""
 import datetime
 import cairo
 import calendar
@@ -35,60 +27,29 @@ import gobject
 import gtk
 from math import pi as PI
 import pango
+from common import *
 
-gdk = gtk.gdk
-
-def check_for_new_month(date):
-    if datetime.date.fromtimestamp(date).day == 1:
-        return True
-    return False
-
-def _in_area(coord_x, coord_y, area):
-    """check if some given X,Y coordinates are within an area.
-    area is either None or a (top_left_x, top_left_y, width, height)-tuple"""
-    if area is None:
-        return False
-    area_x, area_y, area_width, area_height = area
-    return (area_x <= coord_x <= area_x + area_width) and \
-        (area_y <= coord_y <= area_y + area_height)
-
-def get_gtk_rgba(style, palette, i, shade = 1, alpha = 1):
-    """Takes a gtk style and returns a RGB tuple
-
-    Arguments:
-    - style: a gtk_style object
-    - palette: a string representing the palette you want to pull a color from
-        Example: "bg", "fg"
-    - shade: how much you want to shade the color
-    """
-    f = lambda num: (num/65535.0) * shade
-    color = getattr(style, palette)[i]
-    if isinstance(color, gdk.Color):
-        red = f(color.red)
-        green = f(color.green)
-        blue = f(color.blue)
-
-        return (min(red, 1), min(green, 1), min(blue, 1), alpha)
-    else: raise TypeError("Not a valid gdk.Color")
 
 def get_gc_from_colormap(widget, shade):
     """
-    Gets a gdk.GC and modifies the color by shade
+    Gets a gtk.gdk.GC and modifies the color by shade
     """
     gc = widget.style.text_gc[gtk.STATE_INSENSITIVE]
     if gc:
         color = widget.style.text[4]
-        f = lambda num: min((num * shade, 65535.0))
-        color.red = f(color.red)
-        color.green = f(color.green)
-        color.blue = f(color.blue)
+        color = shade_gdk_color(color, shade)
         gc.set_rgb_fg_color(color)
     return gc
 
 
 class CairoHistogram(gtk.DrawingArea):
     """
-    A histogram which is represented by a list of dates, and nitems
+    A histogram which is represented by a list of dates, and nitems.
+
+    There are a few maintenance issues due to the movement abilities. The widget
+    currently is able to capture motion events when the mouse is outside
+    the widget and the button is pressed if it was initially pressed inside
+    the widget. This event mask magic leaves a few flaws open.
     """
     _selected = (0,)
     padding = 2
@@ -107,7 +68,7 @@ class CairoHistogram(gtk.DrawingArea):
     pangofont = None
     _disable_mouse_motion = False
     selected_range = 0
-    _highlighted = tuple()
+    _highlighted = []
     _last_location = -1
     _single_day_only = False
     colors = {
@@ -117,25 +78,19 @@ class CairoHistogram(gtk.DrawingArea):
         "column_selected" : (1, 1, 1, 1),
         "column_alternative" : (1, 1, 1, 1),
         "column_selected_alternative" : (1, 1, 1, 1),
-        "font_color" : (0, 0, 0, 0),
+        "font_color" : "#ffffff",
         "stroke" : (1, 1, 1, 0),
         "shadow" : (1, 1, 1, 0),
         }
 
-    # Today button stuff
-    _today_width = 0
-    _today_text = ""
-    _today_area = None
-    _today_hover = False
+    _store = None
 
-    _datastore = None
     __gsignals__ = {
-        # the index of the first selected item in the datastore.
         "selection-set" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                           (gobject.TYPE_INT,gobject.TYPE_INT)),
+                           (gobject.TYPE_PYOBJECT,)),
         "data-updated" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,()),
         "column_clicked" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                            (gobject.TYPE_INT,))
+                            (gobject.TYPE_PYOBJECT,))
         }
     _connections = {"style-set": "change_style",
                        "expose_event": "_expose",
@@ -143,26 +98,23 @@ class CairoHistogram(gtk.DrawingArea):
                        "motion_notify_event": "mouse_motion_interaction",
                        "key_press_event": "keyboard_interaction",
                        "scroll-event" : "mouse_scroll_interaction",
-                       "selection-set": "check_for_today",
                        }
     _events = (gtk.gdk.KEY_PRESS_MASK | gtk.gdk.BUTTON_MOTION_MASK |
                   gtk.gdk.POINTER_MOTION_HINT_MASK | gtk.gdk.BUTTON_RELEASE_MASK |
                   gtk.gdk.BUTTON_PRESS_MASK)
 
-    def __init__(self, datastore = None, selected_range = 0):
+    def __init__(self):
         """
-        Arguments:
-        - datastore: The.CairoHistograms two dimensional list of dates and nitems
-        - selected_range: the number of days displayed at once
+        :param datastore: The.CairoHistograms two dimensional list of dates and nitems
+        :param selected_range: the number of days displayed at once
         """
         super(CairoHistogram, self).__init__()
+        self._selected = []
         self.set_events(self._events)
         self.set_flags(gtk.CAN_FOCUS)
         for key, val in self._connections.iteritems():
             self.connect(key, getattr(self, val))
         self.font_name = self.style.font_desc.get_family()
-        self.set_datastore(datastore if datastore else [], draw = False)
-        self.selected_range = selected_range
 
     def change_style(self, widget, old_style):
         """
@@ -173,6 +125,10 @@ class CairoHistogram(gtk.DrawingArea):
         self.colors["base"] = get_gtk_rgba(self.style, "base", 0)
         self.colors["column_normal"] =  get_gtk_rgba(self.style, "text", 4, 1.17)
         self.colors["column_selected"] = get_gtk_rgba(self.style, "bg", 3)
+        color = self.style.bg[gtk.STATE_NORMAL]
+        fcolor = self.style.fg[gtk.STATE_NORMAL]
+        self.colors["font_color"] = combine_gdk_color(color, fcolor).to_string()
+
         pal = get_gtk_rgba(self.style, "bg", 3, 1.2)
         self.colors["column_alternative"] = (pal[2], pal[1], pal[0], 1)
         self.colors["column_selected_alternative"] = get_gtk_rgba(self.style, "bg", 3, 0.6)
@@ -184,74 +140,33 @@ class CairoHistogram(gtk.DrawingArea):
         self.bottom_padding = self.font_size + 9 + widget.style.ythickness
         self.gc = get_gc_from_colormap(widget, 0.6)
 
-    def set_selected_range(self, selected_range):
-        """
-        Set the number of days to be colored as selected
-
-        Arguments:
-        - selected_range: the range to be used when setting selected coloring
-        """
-        self.selected_range = selected_range
-        return True
-
-    def set_datastore(self, datastore, draw = True):
-        """
-        Sets the objects datastore attribute using a list
-
-        Arguments:
-        -datastore: A list that is comprised of rows containing
-          a int time and a int nitems
-        """
-        if isinstance(datastore, list):
-            self._datastore = datastore
-            self.largest = 1
-            for date, nitems in self._datastore:
-                if nitems > self.largest: self.largest = nitems
-            if self.largest > self.max_column_height: self.largest = self.max_column_height
-            self.max_width = self.xincrement + (self.xincrement *len(datastore))
+    def set_store(self, store):
+        self._store = store
+        self.largest = min(max(max(map(lambda x: len(x), store.days)), 1), 100)
+        if not self.get_selected():
+            self.set_selected([datetime.date.today()])
         else:
-            raise TypeError("Datastore is not a <list>")
-        self.emit("data-updated")
-        self.set_selected(len(datastore) - self.selected_range)
-
-    def get_datastore(self):
-        return self._datastore
-
-    def prepend_data(self, newdatastore):
-        """
-        Adds the items of a new list before the items of the current datastore
-
-        Arguments:
-        - newdatastore: the new list to be prepended
-
-        ## WARNING SELECTION WILL CHANGE WHEN DOING THIS TO BE FIXED ##
-        """
-        selected = self.get_selected()[-1]
-        self._datastore = newdatastore + self._datastore
+            self.set_selected(self.get_selected())
         self.queue_draw()
-        self.set_selected(len(newdatastore) + selected)
+        self.last_updated = time.time()
+
+    def get_store(self):
+        return self._store
 
     def _expose(self, widget, event):
         """
         The major drawing method that the expose event calls directly
-
-        Arguments:
-        - widget: the widget
-        - event: a gtk event with x and y values
         """
+        widget.style.set_background(widget.window, gtk.STATE_NORMAL)
         context = widget.window.cairo_create()
         self.expose(widget, event, context)
-        if len(self._today_text):
-            self.draw_today(widget, event, context)
 
     def expose(self, widget, event, context):
         """
         The minor drawing method
 
-        Arguments:
-        - widget: the widget
-        - event: a gtk event with x and y values
-        - context: The drawingarea's cairo context from the expose event
+        :param event: a gtk event with x and y values
+        :param context: This drawingarea's cairo context from the expose event
         """
         if not self.pangofont:
             self.pangofont = pango.FontDescription(self.font_name + " %d" % self.font_size)
@@ -260,13 +175,13 @@ class CairoHistogram(gtk.DrawingArea):
             self.gc = get_gc_from_colormap(widget, 0.6)
         context.set_source_rgba(*self.colors["base"])
         context.set_operator(cairo.OPERATOR_SOURCE)
-        context.paint()
+        #context.paint()
         context.rectangle(event.area.x, event.area.y, event.area.width, event.area.height)
         context.clip()
-        context.set_source_rgba(*self.colors["bg"])
-        context.rectangle(event.area.x, event.area.height - self.bottom_padding, event.area.width, event.area.height)
+        #context.set_source_rgba(*self.colors["bg"])
+        context.rectangle(event.area.x, event.area.y, event.area.width, event.area.height - self.bottom_padding)
         context.fill()
-        self.draw_columns_from_datastore(context, event, self.get_selected())
+        self.draw_columns_from_store(context, event, self.get_selected())
         context.set_line_width(1)
         if type(self) == CairoHistogram:
             widget.style.paint_shadow(widget.window, gtk.STATE_NORMAL, gtk.SHADOW_IN,
@@ -276,80 +191,49 @@ class CairoHistogram(gtk.DrawingArea):
             widget.style.paint_focus(widget.window, gtk.STATE_NORMAL, event.area, widget, None, event.area.x, event.area.y,
                                      event.area.width, event.area.height - self.bottom_padding)
 
-    def draw_today(self, widget, event, context):
-        """
-        """
-        layout = widget.create_pango_layout(self._today_text)
-        pangofont = pango.FontDescription(widget.font_name + " %d" % (widget.font_size - 1))
-        if not widget.gc:
-            widget.gc = get_gc_from_colormap(widget, 0.6)
-        layout.set_font_description(pangofont)
-        w, h = layout.get_pixel_size()
-        self._today_width = w + 10
-        self._today_area = (
-            int(event.area.x + event.area.width - self._today_width),
-            int(event.area.height - widget.bottom_padding + 2),
-            self._today_width,
-            widget.bottom_padding - 2)
-        state = gtk.STATE_PRELIGHT
-        shadow = gtk.SHADOW_OUT
-        widget.style.paint_box(
-            widget.window, state, shadow, event.area, widget, "button", *self._today_area)
-        widget.window.draw_layout(
-            widget.gc, int(event.area.x + event.area.width - w -5),
-            int(event.area.height - widget.bottom_padding/2 - h/2), layout)
-
-    def draw_columns_from_datastore(self, context, event, selected):
+    def draw_columns_from_store(self, context, event, selected):
         """
         Draws columns from a datastore
 
-        Arguments:
-        - context: The drawingarea's cairo context from the expose event
-        - event: a gtk event with x and y values
-        - selected: a list of the selected columns
+        :param context: This drawingarea's cairo context from the expose event
+        :param event: a gtk event with x and y values
+        :param selected: a list of the selected dates
         """
         x = self.start_x_padding
         months_positions = []
-        i = 0
-        for date, nitems in self._datastore:
-            if check_for_new_month(date):
-                months_positions += [(date, x)]
-            if len(self._highlighted) > 0 and i >= self._highlighted[0] and i <= self._highlighted[-1] and i in self._highlighted:
-                color = self.colors["column_selected_alternative"] if i in selected else self.colors["column_alternative"]
-            elif not selected:
-                color = self.colors["column_normal"]
-            elif self._single_day_only  and i != selected[-1]:
-                color = self.colors["column_normal"]
-            elif i >= selected[0] and i <= selected[-1] and i in selected:
+        for day in self.get_store().days:
+            if day.date.day == 1:
+                months_positions += [(day.date, x)]
+            if day.date in self._highlighted:
+                color = self.colors["column_selected_alternative"] if day.date in selected else self.colors["column_alternative"]
+            elif day.date in selected:
                 color = self.colors["column_selected"]
             else:
                 color = self.colors["column_normal"]
-            self.draw_column(context, x, event.area.height, nitems, color)
+            self.draw_column(context, x, event.area.height, len(day), color)
             x += self.xincrement
-            i += 1
         if x > event.area.width: # Check for resize
             self.set_size_request(x+self.xincrement, event.area.height)
         for date, xpos in months_positions:
-            self.draw_month(context, xpos - self.padding, event.area.height, date)
+            edge = 0
+            if (date, xpos) == months_positions[-1]:
+                edge = len(self._store)*self.xincrement
+            self.draw_month(context, xpos - self.padding, event.area.height, date, edge)
         self.max_width = x # remove me
 
     def draw_column(self, context, x, maxheight, nitems, color):
         """
         Draws a columns at x with height based on nitems, and maxheight
 
-        Arguments:
-        - context: The drawingarea's cairo context from the expose event
-        - x: The current position in the image
-        - maxheight: The event areas height
-        - nitems: The number of items in the column to be drawn
-        - color: A RGBA tuple
-            Example: (0.3, 0.4, 0.8, 1)
+        :param context: The drawingarea's cairo context from the expose event
+        :param x: The current position in the image
+        :param maxheight: The event areas height
+        :param nitems: The number of items in the column to be drawn
+        :param color: A RGBA tuple Example: (0.3, 0.4, 0.8, 1)
         """
-        if nitems < 2:
-            nitems = 2
-        elif nitems > self.max_column_height:
+        if nitems > self.max_column_height:
             nitems = self.max_column_height
-        maxheight = maxheight - self.bottom_padding
+        maxheight = maxheight - self.bottom_padding - 2
         height = int(((float(nitems)/self.largest)*(maxheight-2))) - self.top_padding
         if height < self.min_column_height:
             height = self.min_column_height
@@ -366,7 +250,7 @@ class CairoHistogram(gtk.DrawingArea):
         context.close_path()
         context.fill()
 
-    def draw_month(self, context, x, height, date):
+    def draw_month(self, context, x, height, date, edge=0):
         """
         Draws a line signifying the start of a month
         """
@@ -375,32 +259,23 @@ class CairoHistogram(gtk.DrawingArea):
         context.move_to(x+self.stroke_offset, 0)
         context.line_to(x+self.stroke_offset, height - self.bottom_padding)
         context.stroke()
-        date = datetime.date.fromtimestamp(date)
         month = calendar.month_name[date.month]
-        date = "%s %d" % (month, date.year)
+        date = "<span color='%s'>%s %d</span>" % (self.colors["font_color"], month, date.year)
         layout = self.create_pango_layout(date)
+        layout.set_markup(date)
         layout.set_font_description(self.pangofont)
         w, h = layout.get_pixel_size()
+        if edge:
+            if x + w > edge: x = edge - w - 5
         self.window.draw_layout(self.gc, int(x + 3), int(height - self.bottom_padding/2 - h/2), layout)
 
-    def set_selected(self, i):
-        """
-        Set the selected items using a int or a list of the selections
-        If you pass this method a int it will select the index + selected_range
-
-        Emits:
-         self._selected[0] and self._selected[-1]
-
-        Arguments:
-        - i: a list or a int where the int will select i + selected_range
-        """
-        if len(self._selected):
-            if i == self._selected[0]:
-                return False
-        if isinstance(i, int):
-            self._selected = range(i, i + self.selected_range)
-            self.emit("selection-set", max(i, 0), max(i + self.selected_range - 1, 0))
-        else: self._selected = (-1,)
+    def set_selected(self, dates):
+        if dates == self._selected:
+            return False
+        self._selected = dates
+        if dates:
+            date = dates[-1]
+        self.emit("selection-set", dates)
         self.queue_draw()
         return True
 
@@ -414,14 +289,14 @@ class CairoHistogram(gtk.DrawingArea):
         """
         clears the selected items
         """
-        self._selected = range(len(self._datastore))[-self.selected_range:]
+        self._selected = []
         self.queue_draw()
 
     def set_highlighted(self, highlighted):
         """
         Sets the widgets which should be highlighted with an alternative color
-        Argument:
-        - highlighted: a list of indexes to be highlighted
+
+        :param highlighted: a list of indexes to be highlighted
         """
         if isinstance(highlighted, list):
             self._highlighted = highlighted
@@ -440,7 +315,7 @@ class CairoHistogram(gtk.DrawingArea):
         self._single_day_only = choice
         self.queue_draw()
 
-    def get_datastore_index_from_cartesian(self, x, y):
+    def get_store_index_from_cartesian(self, x, y):
         """
         Gets the datastore index from a x, y value
         """
@@ -451,18 +326,18 @@ class CairoHistogram(gtk.DrawingArea):
             i = self.get_selected()
             if isinstance(i, list) and len(i) > 0: i = i[-1]
             if event.keyval in (gtk.keysyms.space, gtk.keysyms.Right):
-                i += 1
+                i = i + datetime.timedelta(days=1)
             elif event.keyval in (gtk.keysyms.Left, gtk.keysyms.BackSpace):
-                i -= 1
-            if i < len(self.get_datastore()):
+                i = i + datetime.timedelta(days=-1)
+            if i < datetime.date.today() + datetime.timedelta(days=1):
                 self.change_location(i)
 
     def mouse_motion_interaction(self, widget, event, *args, **kwargs):
         """
         Reacts to mouse moving (while pressed), and clicks
         """
-        #if (event.state == gdk.BUTTON1_MASK and not self._disable_mouse_motion):
-        location = min((self.get_datastore_index_from_cartesian(event.x, event.y), len(self._datastore) - 1))
+        #if (event.state == gtk.gdk.BUTTON1_MASK and not self._disable_mouse_motion):
+        location = min((self.get_store_index_from_cartesian(event.x, event.y), len(self._store.days) - 1))
         if location != self._last_location:
             self.change_location(location)
             self._last_location = location
@@ -473,18 +348,19 @@ class CairoHistogram(gtk.DrawingArea):
         if (event.y > self.get_size_request()[1] - self.bottom_padding and
             event.y < self.get_size_request()[1]):
             return False
-        location = min((self.get_datastore_index_from_cartesian(event.x, event.y), len(self._datastore) - 1))
+        location = min((self.get_store_index_from_cartesian(event.x, event.y), len(self._store.days) - 1))
         if location != self._last_location:
             self.change_location(location)
             self._last_location = location
         return True
 
     def mouse_scroll_interaction(self, widget, event):
-        i = self.get_selected()[-1]
-        if (event.direction in (gdk.SCROLL_UP, gdk.SCROLL_RIGHT)):
-            if i+1< len(self.get_datastore()):
+        date = self.get_selected()[-1]
+        i = self.get_store().dates.index(date)
+        if (event.direction in (gtk.gdk.SCROLL_UP, gtk.gdk.SCROLL_RIGHT)):
+            if i+1< len(self.get_store().days):
                 self.change_location(i+1)
-        elif (event.direction in (gdk.SCROLL_DOWN, gdk.SCROLL_LEFT)):
+        elif (event.direction in (gtk.gdk.SCROLL_DOWN, gtk.gdk.SCROLL_LEFT)):
             if 0 <= i-1:
                 self.change_location(i-1)
 
@@ -492,24 +368,40 @@ class CairoHistogram(gtk.DrawingArea):
         """
         Handles click events
         """
-        if location < 0:
-            return False
-        self.set_selected(max(location - self.selected_range + 1, 0))
-        self.emit("column_clicked", location)
+        if isinstance(location, int):
+            if location < 0:
+                return False
+            store = self.get_store()
+            date = store.days[location].date
+        else: date = location
+        self.emit("column_clicked", date)
         return True
 
-    # Today stuff
-    def check_for_today(self, widget, i, ii):
-        """
-        Changes today to a empty string if the selected item is not today
-        """
-        if ii == len(self.get_datastore())-1:
-            self._today_text = ""
-            self._today_area = None
-        elif len(self._today_text) == 0:
-            self._today_text = _("Today") + " »"
-        self.queue_draw()
-        return True
+    left_icon = get_icon_for_name("back", 16)
+    right_icon = get_icon_for_name("forward", 16)
+    def _expose_scroll_buttons(self, widget, event):
+        render_pixbuf(widget.window, event.area.x + event.area.width-16, event.area.y+event.area.height-16, self.right_icon, False)
+        render_pixbuf(widget.window, event.area.x, event.area.y+event.area.height-16, self.left_icon, False)
+
+
+def _in_area(coord_x, coord_y, area):
+    """check if some given X,Y coordinates are within an area.
+    area is either None or a (top_left_x, top_left_y, width, height)-tuple"""
+    if area is None:
+        return False
+    area_x, area_y, area_width, area_height = area
+    return (area_x <= coord_x <= area_x + area_width) and \
+        (area_y <= coord_y <= area_y + area_height)
+
+
+def _in_area(coord_x, coord_y, area):
+    """check if some given X,Y coordinates are within an area.
+    area is either None or a (top_left_x, top_left_y, width, height)-tuple"""
+    if area is None:
+        return False
+    area_x, area_y, area_width, area_height = area
+    return (area_x <= coord_x <= area_x + area_width) and \
+        (area_y <= coord_y <= area_y + area_height)
 
 
 class TooltipEventBox(gtk.EventBox):
@@ -528,23 +420,22 @@ class TooltipEventBox(gtk.EventBox):
 
     def query_tooltip(self, widget, x, y, keyboard_mode, tooltip):
         if y < self.histogram.get_size_request()[1] - self.histogram.bottom_padding:
-            location = self.histogram.get_datastore_index_from_cartesian(x, y)
+            location = self.histogram.get_store_index_from_cartesian(x, y)
             if location != self._saved_tooltip_location:
                 # don't show the previous tooltip if we moved to another
                 # location
                 self._saved_tooltip_location = location
                 return False
             try:
-                timestamp, count = self.histogram.get_datastore()[location]
+                day = self.histogram.get_store().days[location]
+                count = len(day)
             except IndexError:
                 # there is no bar for at this location
                 # don't show a tooltip
                 return False
-            date = datetime.date.fromtimestamp(timestamp).strftime("%A, %d %B, %Y")
+            date = day.date.strftime("%A, %d %B, %Y")
             tooltip.set_text("%s\n%i %s" % (date, count,
                                             gettext.ngettext("item", "items", count)))
-        elif self.container.histogram._today_text and _in_area(x, y, self.container.histogram._today_area):
-            tooltip.set_text(_("Click today to return to today"))
         else:
             return False
         return True
@@ -587,12 +478,15 @@ class HistogramWidget(gtk.Viewport):
     """
     A container for a CairoHistogram which allows you to scroll
     """
+    __gsignals__ = {
+        # the index of the first selected item in the datastore.
+        "date-changed" : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
+                           (gobject.TYPE_PYOBJECT,)),
+        }
 
-
-    def __init__(self, histo_type, size = (600, 75)):
+    def __init__(self, histo_type=CairoHistogram, size = (600, 75)):
         """
-        Arguments:
-        - histo_type = a CairoHistogram or a derivative
+        :param histo_type: a :class:`CairoHistogram <CairoHistogram>` or a derivative
         """
         super(HistogramWidget, self).__init__()
         self.set_shadow_type(gtk.SHADOW_NONE)
@@ -600,24 +494,35 @@ class HistogramWidget(gtk.Viewport):
         self.eventbox = TooltipEventBox(self.histogram, self)
         self.set_size_request(*size)
         self.add(self.eventbox)
-        self.histogram.connect("button_press_event", self.footer_clicked)
+        self.histogram.connect("column_clicked", self.date_changed)
         self.histogram.connect("selection-set", self.scrubbing_fix)
         self.histogram.queue_draw()
         self.queue_draw()
+        self.connect("size-allocate", self.on_resize)
+        self.histogram.connect("button-press-event", self.mouse_click_scroll)
 
-    def footer_clicked(self, widget, event):
-        """
-        Handles all rejected clicks from bellow the histogram internal view and
-        checks to see if they were inside of the today text
-        """
+    def mouse_click_scroll(self, widget, event):
         hadjustment = self.get_hadjustment()
-        # Check for today button click
-        if (widget._today_text and event.x > hadjustment.value + hadjustment.page_size - widget._today_width):
-            self.histogram.change_location(len(self.histogram.get_datastore()) - 1)
-            return True
-        else:
-            pass # Drag here
-        return False
+        value = hadjustment.get_value()
+        if event.x - value < 16:
+            hadjustment.set_value(max(0, value-10))
+        elif event.x > value + hadjustment.page_size - 16:
+            hadjustment.set_value(min(self.histogram.max_width, value+10))
+        self.histogram.queue_draw()
+
+    def on_resize(self, widget, allocation):
+        dates = self.histogram.get_selected()
+        self.scrubbing_fix(self.histogram, dates)
+
+    def date_changed(self, widget, date):
+        self.emit("date-changed", date)
+
+    def set_store(self, store):
+        self.histogram.set_store(store)
+        self.scroll_to_end()
+
+    def set_dates(self, dates):
+        self.histogram.set_selected(dates)
 
     def scroll_to_end(self, *args, **kwargs):
         """
@@ -627,13 +532,17 @@ class HistogramWidget(gtk.Viewport):
         hadjustment.set_value(1)
         hadjustment.set_value(self.histogram.max_width - hadjustment.page_size)
 
-    def scrubbing_fix(self, widget, i, ii):
+    def scrubbing_fix(self, widget, dates):
         """
         Allows scrubbing to scroll the scroll window
         """
+        if not len(dates):
+            return
+        store = widget.get_store()
+        i = store.dates.index(dates[0])
         hadjustment = self.get_hadjustment()
         proposed_xa = ((i) * self.histogram.xincrement) + self.histogram.start_x_padding
-        proposed_xb = ((i + self.histogram.selected_range) * self.histogram.xincrement) + self.histogram.start_x_padding
+        proposed_xb = ((i + len(dates)) * self.histogram.xincrement) + self.histogram.start_x_padding
         if proposed_xa < hadjustment.value:
             hadjustment.set_value(proposed_xa)
         elif proposed_xb > hadjustment.value + hadjustment.page_size:
